## example calls:
# python grid_export_convert.py rem-3a-gridexporter.txt csv -match_index=11 -token tok_anno -annotations pos lemma inflectionClass inflection -metadata meta::abbr_ddd
# python grid_export_convert.py ridges-3a-gridexporter.txt csv -match_index=6 -token dipl -annotations pos lemma -metadata meta::date meta::place

import os
import csv
import re
import pandas as pd

# Funktion definieren, die match, linken und rechten Kontext extrahiert
def create_context_df(any_list, suffix, match_indices):

    assert len(any_list) == len(match_indices)

    context_dicts = []
    
    for i, match_index in zip(any_list, match_indices):

        i_split = i.split()

        ErgebnisseT = {}
        for hit_idxT, iS in enumerate(i_split):
            if (f'[{match_index}-' in iS) or (f'-{match_index}]' in iS):  # Detect the token with '[11', indicating the hit
                # hit_idxT = i_split.index(iS)

                # Extract the lemma without the index
                token_str = i_split[hit_idxT]
                token_split = token_str.split("[")[0]

                # Extract left and right context without indices
                k_l = [word.split("[")[0] for word in i_split[:hit_idxT]]
                k_l_str = " ".join(k_l)

                k_r = [word.split("[")[0] for word in i_split[hit_idxT+1:]]
                k_r_str = " ".join(k_r)

                # Populate the dictionary with extracted values
                ErgebnisseT[f"match_{suffix}"] = token_split
                ErgebnisseT[f"kontext_links_{suffix}"] = k_l_str
                ErgebnisseT[f"kontext_rechts_{suffix}"] = k_r_str
                
                # Append the dictionary to the list
                context_dicts.append(ErgebnisseT)
        if not ErgebnisseT:
            context_dicts.append({f"match_{suffix}": '', f"kontext_links_{suffix}": '', f"kontext_rechts_{suffix}": ''})

    # Convert the list of dictionaries into a DataFrame
    ctxt_df = pd.DataFrame(context_dicts)

    return ctxt_df


def grid_export_to_df(csv_file, token, used_annotations, meta_data, match_index):

    # Hier wird die csv Datei mit dem GridExplorer-Output eingelesen, 
    doc = csv.reader(csv_file, delimiter='\t') 


    # Hier werden in den for Schleifen zu füllende leere Listen für die zu exportierenden Annotationsebenen erstellt
    dict_of_lists = {anno: [] for anno in [token] + used_annotations + meta_data}

    found_annotations = set()

    #dieser Teil iteriert über die einzelnen Zeilen/Spalten der Tabelle
    for i in doc:
   
        if len(i) == 3: #prüft, ob drei Elemente in der Spalte sind

            col0 = i[0] #definiert Spalte 1
            col1 = i[1] #definiert Spalte 2
            col2 = i[2] #definiert Spalte 3
        
            if not col0 =="":
                nr = col0

            if col1 in dict_of_lists:
                dict_of_lists[col1].append(col2)
                found_annotations.add(col1)
        
        else:
            if found_annotations:
                ## add empty lists for missing annotations
                for annotype in set(dict_of_lists.keys()).difference(found_annotations):
                    print(f'{annotype} fehlt bei Beleg Nr. {nr}')
                    dict_of_lists[annotype].append('')
                found_annotations = set()

    for key in dict_of_lists:
        print(key, len(dict_of_lists[key]))

    # ### Extraktion linker Kontext - Treffer - rechter Kontext


    ## get numbers of elements
    match_indices = []
    for line in dict_of_lists[token]:
        elements = line.split()
        if (f'[{2*match_index - 1}-' in elements[-1]) or (f'-{2*match_index - 1}]' in elements[-1]):
            match_indices.append(match_index)
        else:
            print(f"Achtung: letzer Index nicht {2*match_index - 1}!")
            print(line)
            while True:
                try:
                    user_input = int(input("Bitte match index angeben: "))
                except ValueError:
                    print('Bitte eine Zahl eingeben')
                else:
                    break
            match_indices.append(user_input)


    # Hier werden linker Kontext, match und rechter Kontext für die einzelnen Ebenen extrahiert:
    annotations_token = [token] + used_annotations
    ctxt_dfs = {}
    for layer in annotations_token:
        # extract annotation layer
        ctxt_dfs[layer] = create_context_df(dict_of_lists[layer], layer, match_indices)

    # Hier werden die Metadaten in einen Dataframe gepackt
    meta_df = pd.DataFrame(list(zip(*[dict_of_lists[anno] for anno in meta_data])),
                       columns =meta_data)


    # alles zusammen in ein Dataframe
    result = pd.concat([ctxt_dfs[layer] for layer in annotations_token] + [meta_df], axis =1)

    # Spaltenreihenfolge anpassen
    result = result.reindex(columns=
                            [f'kontext_links_{layer_name}' for layer_name in annotations_token] +
                            [f'match_{layer_name}' for layer_name in annotations_token] +
                            [f'kontext_rechts_{layer_name}' for layer_name in annotations_token] +                                                     
                            meta_data)

    return result



if __name__=='__main__':

    import argparse

    parser = argparse.ArgumentParser(description='Skript zur Umformatierung der Annis GridExport Dateien')
    parser.add_argument('inputfile', type=argparse.FileType('r', encoding='utf-8'))
    parser.add_argument('output', choices=['csv', 'xlsx'])
    parser.add_argument('-match_index', type=int, default=11)
    parser.add_argument('-token', type=str)
    parser.add_argument('-annotations', type=str, nargs='+')
    parser.add_argument('-metadata', type=str, nargs='+')

    args = parser.parse_args()

    if args.annotations is None:
        args.annotations = []
    if args.metadata is None:
        args.metadata = []

    print(args.token)
    print(args.annotations)
    print(args.metadata)

    result = grid_export_to_df(args.inputfile, args.token, args.annotations, args.metadata, args.match_index)

    if args.output == 'csv':

        output_datei = args.inputfile.name[:-4]+ '_.csv'

        result.to_csv(output_datei)

    elif args.output == 'xlsx':
        output_datei = args.inputfile.name[:-4]+ '_.xlsx'

        # Create a Pandas Excel writer using XlsxWriter as the engine.
        writer = pd.ExcelWriter(output_datei, engine='xlsxwriter')

        # Convert the dataframe to an XlsxWriter Excel object.
        result.to_excel(writer, sheet_name='Sheet1',index=False)

        # Close the Pandas Excel writer and output the Excel file.
        writer.close()
